import { actionResult, actionResultError, createRuntime, ErrorTypeEnum, getUniqueKeyFromQpqFunctionRuntime, qpqCoreUtils, QpqRuntimeType, SystemActionType, } from 'quidproquo-core';
import { randomGuid } from '../../../awsLambdaUtils';
export const getDateNow = () => new Date().toISOString();
const getProcessExecuteStory = (qpqConfig) => {
    const moduleName = qpqCoreUtils.getApplicationModuleName(qpqConfig);
    return async (payload, session, actionProcessors, logger, updateSession, dynamicModuleLoader) => {
        let story = await dynamicModuleLoader(payload.runtime);
        if (!story) {
            return actionResultError(ErrorTypeEnum.NotFound, `Unable to dynamically load: [${payload.runtime}]`);
        }
        const functionKey = getUniqueKeyFromQpqFunctionRuntime(payload.runtime);
        const resolveStory = createRuntime(qpqConfig, {
            context: payload.storySession?.context || session.context,
            depth: (payload.storySession?.depth || session.depth || 0) + 1,
            decodedAccessToken: payload.storySession?.decodedAccessToken || session.decodedAccessToken,
            correlation: payload.storySession?.correlation || session.correlation,
        }, async () => actionProcessors, getDateNow, logger, 
        // TODO: Share this logic.
        `${moduleName}::${randomGuid()}`, QpqRuntimeType.EXECUTE_STORY, dynamicModuleLoader, [functionKey]);
        const storyResult = await resolveStory(story, payload.params);
        if (storyResult.error) {
            return actionResultError(storyResult.error.errorType, storyResult.error.errorText, storyResult.error.errorStack ? `${functionKey} -> [${storyResult.error.errorStack}]` : functionKey);
        }
        return actionResult(storyResult.result);
    };
};
export const getSystemExecuteStoryActionProcessor = async (qpqConfig) => ({
    [SystemActionType.ExecuteStory]: getProcessExecuteStory(qpqConfig),
});
